@@ -8,6 +8,7 @@ import pandas as pd
 import google.protobuf.wrappers_pb2 as proto
 import numpy as np
 import google
+import math
 
 def __isTimeDataType(dataType):
     return dataType in frozenset(['TimeColumnConfig', 'StaticTimeConfig'])
@@ -145,6 +146,7 @@ def __makeLineQueryPandas(line_query):
             pass
     return filterExprs
 
+
 def getDatasourceMetaReplyPandas(df, ds_id, ds_name):
     typedColumnConfigsAndMeta = __getColumnConfigsPandas(df)
     valueModifiers = []
@@ -161,14 +163,20 @@ def getDatasourceMetaReplyPandas(df, ds_id, ds_name):
     )
 
 
-# Iterate a dataframe's rows as api.DataRecordsReply
-# with timestamps serialized as unix time
-# mutates the data frame
-def __serializeDataFrame(df, columns, chunk_size = 100):
-    # Partitioning our columns into [string, number, time]
-    project_cols = list(map(lambda c:  c.name, columns))
-    column_types = list(map(lambda c: __columnTypeToString(c), columns))
+def __dataTypeToString(dataType):
+    if __isTimeDataType(dataType):
+        return "Time"
+    if __isNumberDataType(dataType):
+        return "Number"
+    if __isStringDataType(dataType):
+        return "String"
+    raise Exception(f'Unsupprted dataType: {dataType}')
 
+# Iterate a dataframe's rows as api.DataRecordsReply
+# time columns must be serialized as numbers prior to this
+def __serializeDataFrame(df, project_cols, chunk_size = 100):
+    column_configs = map(lambda name: __getColumnTypeOptions(df.dtypes[name], name), project_cols)
+    column_types = list(map(lambda c: __dataTypeToString(c["dataType"]), column_configs))
     string_cols = list(map(lambda x: x[0], filter(lambda c: c[1] == "String", zip(project_cols, column_types))))
     string_dict = dict([(item, index) for (index, item) in enumerate(string_cols)])
 
@@ -178,11 +186,8 @@ def __serializeDataFrame(df, columns, chunk_size = 100):
     time_cols = list(map(lambda x: x[0], filter(lambda c: c[1] == "Time", zip(project_cols, column_types))))
     time_dict = dict([(item, index) for (index, item) in enumerate(time_cols)])
 
-    # @todo: transform times outside
-    # # Transform our time columns into unix timestamps
-    # df[time_cols] = df[time_cols].astype(np.int64)
-
-    for chunk_df in np.array_split(df, chunk_size):
+    num_rows = df.shape[0]
+    for chunk_df in list(filter(lambda x: not x.empty, np.array_split(df, math.ceil(num_rows / chunk_size)))):
         data_records = []
         for _, row in chunk_df.iterrows():
             strings = [api.OptionalString(value=proto.StringValue(value=None))] * len(string_cols)
@@ -191,20 +196,38 @@ def __serializeDataFrame(df, columns, chunk_size = 100):
 
             for c, c_type in zip(project_cols, column_types):
                 if c_type == "String":
-                    strings[string_dict[c]] = api.OptionalString(value=proto.StringValue(value=row[c]))
+                    if row[c] == None:
+                        strings[string_dict[c]] = api.OptionalString(value=None)
+                    else:
+                        strings[string_dict[c]] = api.OptionalString(value=proto.StringValue(value=row[c]))
                 elif c_type == "Number":
-                    numbers[number_dict[c]] = api.OptionalNumber(value=proto.DoubleValue(value=row[c]))
+                    if row[c] == None or math.isnan(row[c]):
+                        numbers[number_dict[c]] = api.OptionalNumber(value=None)
+                    else:
+                        numbers[number_dict[c]] = api.OptionalNumber(value=proto.DoubleValue(value=row[c]))
                 elif c_type == "Time":
-                    times[time_dict[c]] = api.OptionalTime(value=proto.Int64Value(value=row[c]))
+                    if row[c] == None or math.isnan(row[c]):
+                        times[time_dict[c]] = api.OptionalTime(value=None)
+                    else:
+                        times[time_dict[c]] = api.OptionalTime(value=proto.Int64Value(value=row[c]))
             data_records.append(api.DataRecord(strings=strings, numbers=numbers, times=times))
+            print("PUSH RECORD")
+        print("SEND RECORDS")
         yield api.DataRecordsReply(records=data_records)
 
 
-def dataSourceUniquesStreamPandas(df, request):
-    chunk_size = 100  # Determines the number of records per rpc batch
+def dataSourceUniquesStreamPandas(df, request, chunk_size = 100):
     columns = list(request.columns)
     unique_df = df[columns].drop_duplicates()
-    __serializeDataFrame(unique_df, columns, chunk_size)
+
+    # Convert our times to unix timestamps
+    datesOnly = list(
+        unique_df.select_dtypes(include=[np.datetime64]).columns
+    )
+    unique_df[datesOnly] = unique_df[datesOnly].astype(np.int64)
+
+    # Run the serialize code
+    yield from __serializeDataFrame(unique_df, columns, chunk_size)
 
 
 def dataSourceQueryStreamPandas(df, request):
@@ -246,10 +269,20 @@ def dataSourceQueryStreamPandas(df, request):
 
             for c, c_type in zip(project_cols, column_types):
                 if c_type == "String":
-                    strings[string_dict[c]] = api.OptionalString(value=proto.StringValue(value=row[c]))
+                    if row[c] == None:
+                        strings[string_dict[c]] = api.OptionalString(value=None)
+                    else:
+                        strings[string_dict[c]] = api.OptionalString(value=proto.StringValue(value=row[c]))
                 elif c_type == "Number":
-                    numbers[number_dict[c]] = api.OptionalNumber(value=proto.DoubleValue(value=row[c]))
+                    if row[c] == None or math.isnan(row[c]):
+                        numbers[number_dict[c]] = api.OptionalNumber(value=None)
+                    else:
+                        numbers[number_dict[c]] = api.OptionalNumber(value=proto.DoubleValue(value=row[c]))
                 elif c_type == "Time":
-                    times[time_dict[c]] = api.OptionalTime(value=proto.Int64Value(value=row[c]))
+                    if row[c] == None or math.isnan(row[c]):
+                        times[time_dict[c]] = api.OptionalTime(value=None)
+                    else:
+                        times[time_dict[c]] = api.OptionalTime(value=proto.Int64Value(value=row[c]))
+
             data_records.append(api.DataRecord(strings=strings, numbers=numbers, times=times))
         yield api.DataRecordsReply(records=data_records)
